package com.onresolve.jira.groovy.test

import com.atlassian.jira.issue.fields.CustomField
import com.atlassian.jira.issue.customfields.CustomFieldType
import com.atlassian.jira.issue.customfields.CustomFieldSearcher
import com.atlassian.core.util.collection.EasyList
import com.atlassian.jira.issue.context.GlobalIssueContext
import com.atlassian.jira.ComponentManager
import com.atlassian.jira.issue.fields.config.manager.FieldConfigSchemeManager
import com.atlassian.jira.component.ComponentAccessor

import static junit.framework.Assert.assertNotNull
import com.atlassian.jira.bc.project.component.ProjectComponentManager
import com.atlassian.jira.project.version.VersionManager
import com.atlassian.jira.issue.fields.screen.FieldScreenManager
import com.atlassian.jira.issue.index.IssueIndexManager

class CustomFieldBaseTests extends BaseSubTaskTest {

    def fieldConfigSchemeManager = ComponentManager.getComponentInstanceOfType(FieldConfigSchemeManager.class)
    def optionsManager = ComponentAccessor.getOptionsManager()
    def fieldScreenManager = ComponentManager.getComponentInstanceOfType(FieldScreenManager.class)

    CustomField createCustomField(String cfName, String cfKey, String searcherKey) {

        def customField = customFieldManager.getCustomFieldObjectByName(cfName)
        if (customField) {
            return customField
        }

        CustomFieldType customFieldType = customFieldManager.getCustomFieldType(cfKey)
        CustomFieldSearcher customFieldSearcher = customFieldManager.getCustomFieldSearcher(searcherKey)

        customField = customFieldManager.createCustomField(cfName,
            "an eg custom field",
            customFieldType,
            customFieldSearcher,
            EasyList.build(GlobalIssueContext.getInstance()),
            [null]);

        customFieldManager.clear()

        // add to default screen
        def screen = fieldScreenManager.getFieldScreens().find {it.name == "Default Screen"}
        screen.getTabs().first().addFieldScreenLayoutItem(customField.id)


        customField
    }

    public void createIssue() {
        log.debug ("setUp")

        log.debug ("issueType: ${issueType.getName()}")
        assertNotNull(project)

        issue = componentManager.getIssueFactory().getIssue()
        issue.setProject(project.getGenericValue())
        issue.setSummary "my summary"
        issue.setReporterId "admin"
        issue.setIssueType issueType.getGenericValue()
        issue.setPriorityId('2')
        issueManager.createIssue(adminUser, issue);
        log.debug ("Created issue: $issue")
    }
}
