package com.onresolve.jira.groovy.test

import org.junit.Before
import org.junit.Test
import org.apache.log4j.Category
import com.onresolve.jira.groovy.canned.workflow.postfunctions.SendCustomEmail
import com.onresolve.jira.groovy.canned.workflow.conditions.JqlQueryCondition

/**
 * User: jechlin2
 */
class TestJqlQueryCondition extends CustomFieldBaseTests {

    Category log = Category.getInstance(this.class)

    @Before
    public void setUp() {
        tearDown()

        createIssue()
    }

    public void tearDown() {
        deleteAllIssues();
    }

    @Test
    public void testJqlConditionBasic() {
        Map<String, Object> inParams = [
            issue: issue,
            (JqlQueryCondition.FIELD_PREVIEW_ISSUE) : issue.key,
            (JqlQueryCondition.FIELD_JQL_QUERY) : 'issuetype = Bug',
        ] as Map<String, Object>

        def jqlQueryCondition = new JqlQueryCondition()

        def outParams = jqlQueryCondition.doScript(inParams)
        assert outParams['passesCondition'] == true

        inParams[(JqlQueryCondition.FIELD_JQL_QUERY)] = 'issuetype = Task'
        outParams = jqlQueryCondition.doScript(inParams)
        assert outParams['passesCondition'] == false

        // invalid query
        inParams[(JqlQueryCondition.FIELD_JQL_QUERY)] = 'issuetype = xTask'
        outParams = jqlQueryCondition.doScript(inParams)
        assert outParams['passesCondition'] == false

        assert "The value 'xTask' does not exist for the field 'issuetype'." == jqlQueryCondition.doValidate(inParams, true).getErrors().get(JqlQueryCondition.FIELD_JQL_QUERY)
    }
}